@@ -5,7 +5,7 @@ import sys
 
 from . import maybe_patch_concurrency
 
-__all__ = ['main']
+__all__ = ('main',)
 
 
 def main():
