@@ -10,7 +10,7 @@ from celery.utils.log import get_logger
 
 from . import state
 
-__all__ = ['asynloop', 'synloop']
+__all__ = ('asynloop', 'synloop')
 
 # pylint: disable=redefined-outer-name
 # We cache globals and attribute lookups, so disable this warning.
